@@ -236,6 +236,15 @@ doCmd(char const *sock, char const *cmd, int decode)
 	percent_decode((unsigned char *)ans);
     }
     printf("%s", ans);
+
+    /* If it didn't end with a newline, add one */
+    size_t l = strlen(ans);
+    if (l > 0) {
+	if (ans[l-1] != '\n') {
+	    printf("\n");
+	}
+    }
+
     return EXIT_SUCCESS;
 }
 
@@ -260,9 +269,9 @@ doStatus(char const *sock)
     *s = 0;
 
     l = strlen(ans);
-    printf("Max slaves: %d\n", l);
+    printf("Max workers: %d\n", l);
     for (i=0; i<l; i++) {
-	printf("Slave %d: ", i);
+	printf("Worker %d: ", i);
 	switch(ans[i]) {
 	case 'I':
 	    printf("idle\n");
@@ -307,7 +316,7 @@ doBarStatus(char const *sock)
     nqueued = 0;
     maxqueue = 0;
 
-    /* Count number of busy slaves */
+    /* Count number of busy workers */
     s = ans;
     while (*s && *s != ' ') {
 	if (*s == 'B') {
@@ -318,7 +327,7 @@ doBarStatus(char const *sock)
     }
 
     if (ntotal == 0) {
-	printf("Max 0 slaves ??\n");
+	printf("Max 0 workers ??\n");
 	return EXIT_SUCCESS;
     }
 
@@ -408,7 +417,7 @@ doHLoad(char const *rawcmd,
     if (secs_24) mps_24 = (double) msgs_24 / (double) secs_24;
     else        mps_24 = 0;
 
-    printf("%6s %17s %15s %15s %s\n", "Load", msgstr, persec, ms, "  Avg Busy Slaves");
+    printf("%6s %17s %15s %15s %s\n", "Load", msgstr, persec, ms, "  Avg Busy Workers");
     printf("%6s %15d %15.2f %15.1f %15.2f\n", " 1h", msgs_1, mps_1, ams_1, avg_1);
     printf("%6s %15d %15.2f %15.1f %15.2f\n", " 4h", msgs_4, mps_4, ams_4, avg_4);
     printf("%6s %15d %15.2f %15.1f %15.2f\n", "12h", msgs_12, mps_12, ams_12, avg_12);
@@ -425,7 +434,7 @@ doLoad1(char const *sock,
     int num_scans, num_relayoks, num_senderoks, num_recipoks;
     double avg_scans, avg_relayoks, avg_senderoks, avg_recipoks;
     double ms_scans, ms_relayoks, ms_senderoks, ms_recipoks;
-    int busy_slaves, idle_slaves, stopped_slaves, killed_slaves;
+    int busy_workers, idle_workers, stopped_workers, killed_workers;
     int msgs_processed, activations, queue_size, queued_requests, uptime, back;
     int cscan, crelayok, csenderok, crecipok;
 
@@ -437,19 +446,19 @@ doLoad1(char const *sock,
 	       &num_relayoks, &avg_relayoks, &ms_relayoks,
 	       &num_senderoks, &avg_senderoks, &ms_senderoks,
 	       &num_recipoks, &avg_recipoks, &ms_recipoks,
-	       &busy_slaves, &idle_slaves, &stopped_slaves, &killed_slaves,
+	       &busy_workers, &idle_workers, &stopped_workers, &killed_workers,
 	       &msgs_processed, &activations, &queue_size, &queued_requests, &uptime, &back, &cscan, &crelayok, &csenderok, &crecipok) != 26) {
 	printf("Could not interpret response: %s", ans);
 	return EXIT_FAILURE;
     }
 
     printf("Statistics for last %d seconds:\n", back);
-    printf("%10s %15s %15s %15s %15s\n", "Cmd", "Num", "Num/s", "AvgSlaves", "AvgMS");
+    printf("%10s %15s %15s %15s %15s\n", "Cmd", "Num", "Num/s", "AvgWorkers", "AvgMS");
     printf("%10s %15d %15.1f %15.1f %15.1f\n", "Scan", num_scans, (double) num_scans / (double) back, avg_scans, ms_scans);
     printf("%10s %15d %15.1f %15.1f %15.1f\n", "Relayok", num_relayoks, (double) num_relayoks / (double) back, avg_relayoks, ms_relayoks);
     printf("%10s %15d %15.1f %15.1f %15.1f\n", "Senderok", num_senderoks, (double) num_senderoks / (double) back, avg_senderoks, ms_senderoks);
     printf("%10s %15d %15.1f %15.1f %15.1f\n", "Recipok", num_recipoks, (double) num_recipoks / (double) back, avg_recipoks, ms_recipoks);
-    printf("\nCurrent Snapshot:\nSlaves: busy=%d idle=%d stopped=%d killed=%d\n", busy_slaves, idle_slaves, stopped_slaves, killed_slaves);
+    printf("\nCurrent Snapshot:\nWorkers: busy=%d idle=%d stopped=%d killed=%d\n", busy_workers, idle_workers, stopped_workers, killed_workers);
     printf("Cmds:   scan=%d relayok=%d senderok=%d recipok=%d\n", cscan, crelayok, csenderok, crecipok);
     printf("Queue:  size=%d queued=%d\n", queue_size, queued_requests);
     return EXIT_SUCCESS;
@@ -486,7 +495,7 @@ doLoad(char const *rawcmd,
     mps_5 = (double) msgs_5 / (5*60.0);
     mps_10 = (double) msgs_10 / (10*60.0);
 
-    printf("%6s %17s %15s %15s %s\n", "Load", msgstr, persec, ms, "  Avg Busy Slaves");
+    printf("%6s %17s %15s %15s %s\n", "Load", msgstr, persec, ms, "  Avg Busy Workers");
     printf("%6s %15d %15.2f %15.1f %15.2f\n", "10 Sec", msgs_0, mps_0, ams_0, avg_0);
     printf("%6s %15d %15.2f %15.1f %15.2f\n", "1 Min", msgs_1, mps_1, ams_1, avg_1);
     printf("%6s %15d %15.2f %15.1f %15.2f\n", "5 Min", msgs_5, mps_5, ams_5, avg_5);
