@@ -109,6 +109,8 @@ extern int find_syslog_facility(char const *facility_name);
 #define MD_SMFI_TRY(func, args) do { if (func args != MI_SUCCESS) syslog(LOG_WARNING, "%s: %s returned MI_FAILURE", data->qid, #func); } while (0)
 
 char *scan_body = NULL;
+static char *pidfile = NULL;
+static char *lockfile = NULL;
 
 #define KEY_FILE CONFDIR "/mimedefang-ip-key"
 
@@ -250,8 +252,8 @@ static int doRecipientCheck = 0;
 /* Keep directories around if multiplexor fails? */
 static int keepFailedDirectories = 0;
 
-/* Number of scanning slaves reserved for connection from loopback */
-static int slavesReservedForLoopback = -1;
+/* Number of scanning workers reserved for connection from loopback */
+static int workersReservedForLoopback = -1;
 
 static void set_dsn(SMFICTX *ctx, char *buf2, int code);
 
@@ -473,25 +475,6 @@ set_reply(SMFICTX *ctx,
     return retcode;
 }
 
-/**********************************************************************
-*%FUNCTION: closefiles
-*%ARGUMENTS:
-* notthis - file descriptor that should not be closed
-*%RETURNS:
-* Nothing
-*%DESCRIPTION:
-* Hack -- closes a whole bunch of file descriptors.  Use after a fork()
-*         to conserve descriptors.
-***********************************************************************/
-static void
-closefiles(int notthis)
-{
-    int i;
-    for (i=0; i<CLOSEFDS; i++) {
-	if (i != notthis) (void) close(i);
-    }
-}
-
 /**********************************************************************
 *%FUNCTION: mfconnect
 *%ARGUMENTS:
@@ -521,13 +504,13 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
     mfclose(ctx);
 
     /* If too many running filters, reject connection at this phase.
-       However, if slavesReservedForLoopback >= 0, accept or queue
+       However, if workersReservedForLoopback >= 0, accept or queue
        connections from localhost
      */
     if (!AllowNewConnectionsToQueue) {
 	int is_local;
-	int required_slaves;
-	int n = MXCheckFreeSlaves(MultiplexorSocketName, NULL);
+	int required_workers;
+	int n = MXCheckFreeWorkers(MultiplexorSocketName, NULL);
 	if (n < 0) {
 	    syslog(LOG_WARNING, "mfconnect: Error communicating with multiplexor");
 	    DEBUG_EXIT("mfconnect", "SMFIS_TEMPFAIL");
@@ -539,19 +522,19 @@ mfconnect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *sa)
 	    is_local = is_localhost(sa);
 	}
 
-	if (slavesReservedForLoopback >= 0) {
+	if (workersReservedForLoopback >= 0) {
 	    if (is_local) {
-		required_slaves = 0;
+		required_workers = 0;
 	    } else {
-		required_slaves = slavesReservedForLoopback;
+		required_workers = workersReservedForLoopback;
 	    }
 	} else {
-	    required_slaves = 0;
+	    required_workers = 0;
 	}
-	if (n <= required_slaves) {
-	    if (slavesReservedForLoopback < 0 ||
+	if (n <= required_workers) {
+	    if (workersReservedForLoopback < 0 ||
 		! is_local) {
-		syslog(LOG_WARNING, "mfconnect: No free slaves: Need %d, found %d", required_slaves+1, n);
+		syslog(LOG_WARNING, "mfconnect: No free workers: Need %d, found %d", required_workers+1, n);
 		DEBUG_EXIT("mfconnect", "SMFIS_TEMPFAIL");
 		return SMFIS_TEMPFAIL;
 	    }
@@ -853,6 +836,7 @@ envfrom(SMFICTX *ctx, char **from)
     if (data->qid && data->qid != NOQUEUE) {
 	free(data->qid);
 	data->qid = NOQUEUE;
+	data->qid_written = 0;
     }
     if (queueid && *queueid) {
 	data->qid = strdup_with_log(queueid);
@@ -2286,6 +2270,7 @@ usage(void)
     fprintf(stderr, "  -t                -- Do recipient checks before processing body\n");
     fprintf(stderr, "  -q                -- Allow new connections to be queued by multiplexor\n");
     fprintf(stderr, "  -P file           -- Write process-ID of daemon to specified file\n");
+    fprintf(stderr, "  -o file           -- Use specified file as a lock file\n");
     fprintf(stderr, "  -T                -- Log filter times to syslog\n");
     fprintf(stderr, "  -b n              -- Set listen() backlog to n\n");
     fprintf(stderr, "  -C                -- Try very hard to conserve file descriptors\n");
@@ -2297,7 +2282,7 @@ usage(void)
     fprintf(stderr, "  -L ip.addr        -- Specify 'equivalent-to-loopback' address\n");
     fprintf(stderr, "  -H                -- Do HELO checks before processing any messages\n");
     fprintf(stderr, "  -c                -- Strip bare <CR> characters from message body\n");
-    fprintf(stderr, "  -R num            -- Reserve num slaves for connections from localhost\n");
+    fprintf(stderr, "  -R num            -- Reserve num workers for connections from localhost\n");
     fprintf(stderr, "  -G                -- Make sockets group-writable and files group-readable\n");
     fprintf(stderr, "  -N                -- Do not pass rejected recipients to milter (Sendmail\n");
     fprintf(stderr, "                       8.14.0 and newer only.)\n");
@@ -2305,7 +2290,8 @@ usage(void)
     exit(EXIT_FAILURE);
 }
 
-#define REPORT_FAILURE(msg, len) do { if (kidpipe[1] >= 0) { write(kidpipe[1], "E" msg, len+1); } else { fprintf(stderr, "%s\n", msg); } } while(0)
+#define REPORT_FAILURE(msg) do { if (kidpipe[1] >= 0) { write(kidpipe[1], "E" msg, strlen(msg)+1); } else { fprintf(stderr, "%s\n", msg); } } while(0)
+
 /**********************************************************************
 * %FUNCTION: main
 * %ARGUMENTS:
@@ -2321,7 +2307,6 @@ main(int argc, char **argv)
     int c;
     int mx_alive;
     pid_t i;
-    char *pidfile = NULL;
     struct passwd *pw = NULL;
     FILE *fp;
     int facility = LOG_MAIL;
@@ -2330,7 +2315,10 @@ main(int argc, char **argv)
     int got_p_option = 0;
     int kidpipe[2];
     char kidmsg[256];
-
+    int pidfile_fd = -1;
+    int lockfile_fd = -1;
+    int rc;
+    int j;
     mode_t socket_umask = 077;
     mode_t file_umask   = 077;
 
@@ -2387,7 +2375,7 @@ main(int argc, char **argv)
     }
 
     /* Process command line options */
-    while ((c = getopt(argc, argv, "GNCDHL:MP:R:S:TU:Xa:b:cdhkm:p:qrstvx:z:y")) != -1) {
+    while ((c = getopt(argc, argv, "GNCDHL:MP:o:R:S:TU:Xa:b:cdhkm:p:qrstvx:z:y")) != -1) {
 	switch (c) {
 	case 'y':
 	    setsymlist_ok = 1;
@@ -2406,9 +2394,9 @@ main(int argc, char **argv)
 #endif
 	    break;
 	case 'R':
-	    sscanf(optarg, "%d", &slavesReservedForLoopback);
-	    if (slavesReservedForLoopback < -1) {
-		slavesReservedForLoopback = -1;
+	    sscanf(optarg, "%d", &workersReservedForLoopback);
+	    if (workersReservedForLoopback < -1) {
+		workersReservedForLoopback = -1;
 	    }
 	    break;
 	case 'H':
@@ -2520,6 +2508,16 @@ main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	    }
 	    break;
+	case 'o':
+	    /* Use this as our lock file */
+	    if (lockfile != NULL) free(lockfile);
+
+	    lockfile = strdup(optarg);
+	    if (!lockfile) {
+		fprintf(stderr, "%s: Out of memory\n", argv[0]);
+		exit(EXIT_FAILURE);
+	    }
+	    break;
 	case 'P':
 	    /* Write our pid to this file */
 	    if (pidfile != NULL) free(pidfile);
@@ -2610,6 +2608,17 @@ main(int argc, char **argv)
 	exit(EXIT_FAILURE);
     }
 
+    /* Open the pidfile as root.  We'll write the pid later on in the grandchild */
+    if (pidfile) {
+	pidfile_fd = open(pidfile, O_RDWR|O_CREAT, 0666);
+	if (pidfile_fd < 0) {
+	    syslog(LOG_ERR, "Could not open PID file %s: %m", pidfile);
+	    exit(EXIT_FAILURE);
+	}
+	/* It needs to be world-readable */
+	fchmod(pidfile_fd, 0644);
+    }
+
     /* Look up user */
     if (user) {
 	pw = getpwnam(user);
@@ -2654,6 +2663,8 @@ main(int argc, char **argv)
 	exit(EXIT_FAILURE);
     }
 
+    (void) closelog();
+
     /* Daemonize */
     if (!nodaemon) {
 	/* Set up a pipe so child can report back when it's happy */
@@ -2675,7 +2686,7 @@ main(int argc, char **argv)
 	    if (i < 0) {
 		fprintf(stderr, "Error reading message from child: %s\n",
 			strerror(errno));
-		exit(1);
+		exit(EXIT_FAILURE);
 	    }
 	    /* Zero-terminate the string */
 	    kidmsg[i] = 0;
@@ -2699,9 +2710,7 @@ main(int argc, char **argv)
 	signal(SIGHUP, SIG_IGN);
 	i = fork();
 	if (i < 0) {
-	    fprintf(stderr, "%s: fork() failed\n", argv[0]);
-	    /* Signal the waiting parent */
-	    write(kidpipe[1], "Efork() failed", 14);
+	    REPORT_FAILURE("fork() failed");
 	    exit(EXIT_FAILURE);
 	} else if (i != 0) {
 	    exit(EXIT_SUCCESS);
@@ -2712,23 +2721,32 @@ main(int argc, char **argv)
 	kidpipe[1] = -1;
     }
 
-    /* Write pid */
-    if (pidfile) {
-	if (write_and_lock_pidfile(pidfile) < 0) {
-	    /* Signal the waiting parent */
-	    REPORT_FAILURE("Cannot lock pidfile: Is another copy running?", 45);
-	    exit(1);
+    /* In the actual daemon */
+    for (j=0; j<CLOSEFDS; j++) {
+	/* If we are not a daemon, leave stdin/stdout/stderr open */
+	if (nodaemon && j < 3) {
+	    continue;
+	}
+	if (j != pidfile_fd && j != kidpipe[1]) {
+	    close(j);
 	}
-	free(pidfile);
     }
 
-    (void) closelog();
-    closefiles(kidpipe[1]);
+    /* Do the locking */
+    if (pidfile || lockfile) {
+	if ( (lockfile_fd = write_and_lock_pidfile(pidfile, lockfile, pidfile_fd)) < 0) {
+	    /* Signal the waiting parent */
+	    REPORT_FAILURE("Cannot lock lockfile: Is another copy running?");
+	    exit(EXIT_FAILURE);
+	}
+    }
 
     /* Direct stdin/stdout/stderr to /dev/null */
-    open("/dev/null", O_RDWR);
-    open("/dev/null", O_RDWR);
-    open("/dev/null", O_RDWR);
+    if (!nodaemon) {
+	open("/dev/null", O_RDWR);
+	open("/dev/null", O_RDWR);
+	open("/dev/null", O_RDWR);
+    }
 
     openlog("mimedefang", LOG_PID, facility);
 
@@ -2745,7 +2763,7 @@ main(int argc, char **argv)
     if (ValidateHeader[0]) {
 	syslog(LOG_DEBUG, "IP validation header is %s", ValidateHeader);
     }
-    syslog(LOG_INFO, "MIMEDefang alive. slavesReservedForLoopback=%d AllowNewConnectionsToQueue=%d doRelayCheck=%d doHeloCheck=%d doSenderCheck=%d doRecipientCheck=%d", slavesReservedForLoopback, AllowNewConnectionsToQueue, doRelayCheck, doHeloCheck, doSenderCheck, doRecipientCheck);
+    syslog(LOG_INFO, "MIMEDefang alive. workersReservedForLoopback=%d AllowNewConnectionsToQueue=%d doRelayCheck=%d doHeloCheck=%d doSenderCheck=%d doRecipientCheck=%d", workersReservedForLoopback, AllowNewConnectionsToQueue, doRelayCheck, doHeloCheck, doSenderCheck, doRecipientCheck);
 
 #ifdef ENABLE_DEBUGGING
     signal(SIGSEGV, handle_sig);
@@ -2758,7 +2776,7 @@ main(int argc, char **argv)
 	struct timeval sleeptime;
 	sleeptime.tv_sec = 0;
 	sleeptime.tv_usec = 200000;
-	if (MXCheckFreeSlaves(MultiplexorSocketName, NULL) >= 0) {
+	if (MXCheckFreeWorkers(MultiplexorSocketName, NULL) >= 0) {
 	    mx_alive = 1;
 	    break;
 	}
@@ -2769,7 +2787,9 @@ main(int argc, char **argv)
 	syslog(LOG_INFO, "Multiplexor alive - entering main loop");
     } else {
 	/* Signal the waiting parent */
-	REPORT_FAILURE("Multiplexor socket did not appear.  Exiting.", 44);
+	REPORT_FAILURE("Multiplexor socket did not appear.  Exiting.");
+	if (pidfile) unlink(pidfile);
+	if (lockfile) unlink(lockfile);
 	exit(EXIT_FAILURE);
     }
 
@@ -2778,7 +2798,14 @@ main(int argc, char **argv)
 	write(kidpipe[1], "X", 1);
 	close(kidpipe[1]);
     }
-    return smfi_main();
+    rc = (int) smfi_main();
+    if (pidfile) {
+	unlink(pidfile);
+    }
+    if (lockfile) {
+	unlink(lockfile);
+    }
+    return rc;
 }
 
 /**********************************************************************
