@@ -315,25 +315,25 @@ MXCommand(char const *sockname,
 }
 
 /**********************************************************************
-* %FUNCTION: MXCheckFreeSlaves
+* %FUNCTION: MXCheckFreeWorkers
 * %ARGUMENTS:
 *  sockname -- MX socket name
 * %RETURNS:
-*  >0 if there are free slaves, 0 if all slaves are busy, -1 if there
+*  >0 if there are free workers, 0 if all workers are busy, -1 if there
 *  was an error.
 * %DESCRIPTION:
-*  Queries multiplexor for number of free slaves.
+*  Queries multiplexor for number of free workers.
 ***********************************************************************/
 int
-MXCheckFreeSlaves(char const *sockname, char const *qid)
+MXCheckFreeWorkers(char const *sockname, char const *qid)
 {
     char ans[SMALLBUF];
-    int slaves;
+    int workers;
 
     if (MXCommand(sockname, "free\n", ans, SMALLBUF-1, qid) < 0) return MD_TEMPFAIL;
 
-    if (sscanf(ans, "%d", &slaves) != 1) return MD_TEMPFAIL;
-    return slaves;
+    if (sscanf(ans, "%d", &workers) != 1) return MD_TEMPFAIL;
+    return workers;
 }
 
 /**********************************************************************
@@ -1300,30 +1300,53 @@ free_debug(void *ctx, void *x, char const *fname, int line)
 #endif
 
 int
-write_and_lock_pidfile(char const *pidfile)
+write_and_lock_pidfile(char const *pidfile, char *lockfile, int pidfile_fd)
 {
-    int fd;
     struct flock fl;
     char buf[64];
+    int lockfile_fd;
+    size_t len;
+
+    if (!lockfile) {
+	if (!pidfile) {
+	    return -1;
+	}
+	len = strlen(pidfile) + 6;
+	/* If no lockfile was supplied, construct one based on pidfile */
+	lockfile = malloc(len);
+	if (!lockfile) {
+	    return -1;
+	}
+
+	snprintf(lockfile, len, "%s.lock", pidfile);
+    }
+
+    lockfile_fd = open(lockfile, O_RDWR|O_CREAT, 0666);
+    if (lockfile_fd < 0) {
+	return -1;
+    }
 
     fl.l_type = F_WRLCK;
     fl.l_whence = SEEK_SET;
     fl.l_start = 0;
     fl.l_len = 0;
 
-    fd = open(pidfile, O_RDWR|O_CREAT, 0666);
-    if (fd < 0) {
-	syslog(LOG_ERR, "Could not open PID file %s: %m", pidfile);
+    if (fcntl(lockfile_fd, F_SETLK, &fl) < 0) {
+	syslog(LOG_ERR, "Could not lock lockfile file %s: %m.  Is another copy running?", lockfile);
 	return -1;
     }
-    if (fcntl(fd, F_SETLK, &fl) < 0) {
-	syslog(LOG_ERR, "Could not lock PID file %s: %m.  Is another copy running?", pidfile);
-	return -1;
+    if (pidfile_fd >= 0) {
+	ftruncate(pidfile_fd, 0);
+	snprintf(buf, sizeof(buf), "%lu\n", (unsigned long) getpid());
+	write(pidfile_fd, buf, strlen(buf));
+
+	/* Close the pidfile fd; no longer needed */
+	if (close(pidfile_fd) < 0) {
+	    return -1;
+	}
     }
-    ftruncate(fd, 0);
-    snprintf(buf, sizeof(buf), "%lu\n", (unsigned long) getpid());
-    write(fd, buf, strlen(buf));
-    /* Do NOT close fd... it will close and lock will be released
+
+    /* Do NOT close lockfile_fd... it will close and lock will be released
        when we exit */
-    return 0;
+    return lockfile_fd;
 }
