@@ -11,8 +11,8 @@ exec wish "$0" -- "$@" ; clear; echo "*****"; echo "Cannot find 'wish' -- you ne
 # Main update interval (ms)
 set MainUpdateInterval 1500
 
-# Busy slave update interval (ms)
-set BusySlaveUpdateInterval 3000
+# Busy worker update interval (ms)
+set BusyWorkerUpdateInterval 3000
 
 # Trace command - pid is appended
 set TraceCommand "strace -s 100 -t -p"
@@ -108,8 +108,8 @@ proc del_machine { mach } {
 	    catch { unset Data($mach,persec_snap) }
 	    catch { unset Data($mach,time_snap) }
 	    catch { unset Data($mach,qsize) }
-	    catch { unset Data($mach,busyslavewin) }
-	    catch { unset Data($mach,busyslaveafter) }
+	    catch { unset Data($mach,busyworkerwin) }
+	    catch { unset Data($mach,busyworkerafter) }
 	    catch { unset Data($mach,error) }
 	    set did_something 1
 	    continue
@@ -233,14 +233,14 @@ proc mach_populate_data { mach index line } {
     }
 
     foreach { msg0 msg1 msg5 msg10 busy0 busy1 busy5 busy10 ms0 ms1 ms5 ms10 a0 a1 a5 a10 r0 r1 r5 r10 busy idle stopped killed msgs activations qsize qnum uptime} $line { break }
-    set total_slaves [expr $busy + $idle + $stopped + $killed]
+    set total_workers [expr $busy + $idle + $stopped + $killed]
     set ms0 [format "%.0f" $ms0]
     set msg0 [format "%.2f" [expr $msg0 / 10.0]]
 
     lappend Data($mach,busy) $busy0
     lappend Data($mach,time) $ms0
     lappend Data($mach,persec) $msg0
-    set Data($mach,total_slaves) $total_slaves
+    set Data($mach,total_workers) $total_workers
     set Data($mach,busy_snap) $busy
     set Data($mach,persec_snap) $msg0
     set Data($mach,time_snap) $ms0
@@ -250,14 +250,14 @@ proc mach_populate_data { mach index line } {
 
 proc mach_populate_data_new_style { mach index line } {
     global Data
-    foreach { scans avgbusyscans scantime relayoks avgbusyrelayoks relayoktime senderoks avgbusysenderoks senderoktime recipoks avgbusyrecipoks recipoktime busyslaves idleslaves stoppedslaves killedslaves msgs activations qsize numqueued uptime back } $line { break }
+    foreach { scans avgbusyscans scantime relayoks avgbusyrelayoks relayoktime senderoks avgbusysenderoks senderoktime recipoks avgbusyrecipoks recipoktime busyworkers idleworkers stoppedworkers killedworkers msgs activations qsize numqueued uptime back } $line { break }
 
     set scantime [format "%.0f" $scantime]
     set relayoktime [format "%.0f" $relayoktime]
     set senderoktime [format "%.0f" $senderoktime]
     set recipoktime [format "%.0f" $recipoktime]
 
-    set total_slaves [expr $busyslaves + $idleslaves + $stoppedslaves + $killedslaves]
+    set total_workers [expr $busyworkers + $idleworkers + $stoppedworkers + $killedworkers]
 
     set back [expr 1.0 * $back]
     set scanspersec [expr (1.0 * $scans) / $back ]
@@ -270,7 +270,7 @@ proc mach_populate_data_new_style { mach index line } {
     set senderokspersec [format "%.2f" $senderokspersec]
     set recipokspersec [format "%.2f" $recipokspersec]
 
-    lappend Data($mach,busy) $busyslaves
+    lappend Data($mach,busy) $busyworkers
     lappend Data($mach,scantime) $scantime
     lappend Data($mach,scanspersec) $scanspersec
     lappend Data($mach,relayoktime) $relayoktime
@@ -280,8 +280,8 @@ proc mach_populate_data_new_style { mach index line } {
     lappend Data($mach,recipoktime) $recipoktime
     lappend Data($mach,recipokspersec) $recipokspersec
 
-    set Data($mach,total_slaves) $total_slaves
-    set Data($mach,busy_snap) $busyslaves
+    set Data($mach,total_workers) $total_workers
+    set Data($mach,busy_snap) $busyworkers
     set Data($mach,scanspersec_snap) $scanspersec
     set Data($mach,relayokspersec_snap) $relayokspersec
     set Data($mach,senderokspersec_snap) $senderokspersec
@@ -322,10 +322,10 @@ proc redraw_new_style {} {
     set relayoktime_total 0
     set senderoktime_total 0
     set recipoktime_total 0
-    set busyslaves_total 0
-    set totalslaves_total 0
-    set busyslaves_active 0
-    set totalslaves_active 0
+    set busyworkers_total 0
+    set totalworkers_total 0
+    set busyworkers_active 0
+    set totalworkers_active 0
     set num_machines 0
 
     set num_graphs 1
@@ -346,7 +346,7 @@ proc redraw_new_style {} {
     .top.clabels delete all
     set spacing [expr $wid / (1.0 * $num_graphs)]
     set x [expr $spacing / 2.0]
-    .top.clabels create text $x 2 -anchor n -fill "#A00000" -text "Busy Slaves"
+    .top.clabels create text $x 2 -anchor n -fill "#A00000" -text "Busy Workers"
     if {$NewStyleShowScans} {
 	set x [expr $x + $spacing]
 	.top.clabels create text $x 2 -anchor n -fill "#00A000" -text "Scans/d"
@@ -385,16 +385,16 @@ proc redraw_new_style {} {
 
 	# Update totals
 	set busy $Data($mach,busy_snap)
-	set totalslaves $Data($mach,total_slaves)
+	set totalworkers $Data($mach,total_workers)
 
-	# Only update slave counts for machines that are actually doing something
+	# Only update worker counts for machines that are actually doing something
 	if {$Data($mach,scanspersec_snap) > 0 || $Data($mach,relayokspersec_snap) > 0 || $Data($mach,senderokspersec_snap) > 0 || $Data($mach,recipokspersec_snap) > 0} {
-	    set totalslaves_active [expr $totalslaves_active + $totalslaves]
-	    set busyslaves_active [expr $busyslaves_active + 1.0*$busy]
+	    set totalworkers_active [expr $totalworkers_active + $totalworkers]
+	    set busyworkers_active [expr $busyworkers_active + 1.0*$busy]
 	}
 
-	set totalslaves_total [expr $totalslaves_total + $totalslaves]
-	set busyslaves_total [expr $busyslaves_total + 1.0*$busy]
+	set totalworkers_total [expr $totalworkers_total + $totalworkers]
+	set busyworkers_total [expr $busyworkers_total + 1.0*$busy]
 
 	set scanspersec_total [expr $scanspersec_total + 1.0*$Data($mach,scanspersec_snap)]
 	set scantime_total [expr $scantime_total + (1.0*$Data($mach,scanspersec_snap) * $Data($mach,scantime_snap))]
@@ -406,9 +406,9 @@ proc redraw_new_style {} {
 	set recipoktime_total [expr $recipoktime_total + (1.0*$Data($mach,recipokspersec_snap) * $Data($mach,recipoktime_snap))]
 
 	set graph 0
-	set Data($mach,busy) [graph [expr $graph / (1.0 * $num_graphs)] [expr ($graph + 1.0) / (1.0 * $num_graphs)] 0 $Data($mach,total_slaves) $Data($mach,busy) $index "Busy" $graph "#A00000" 1]
-	if {$totalslaves > 0} {
-	    set pctbusy [expr int((1.0*$busy) / (1.0*$totalslaves) * 100)]
+	set Data($mach,busy) [graph [expr $graph / (1.0 * $num_graphs)] [expr ($graph + 1.0) / (1.0 * $num_graphs)] 0 $Data($mach,total_workers) $Data($mach,busy) $index "Busy" $graph "#A00000" 1]
+	if {$totalworkers > 0} {
+	    set pctbusy [expr int((1.0*$busy) / (1.0*$totalworkers) * 100)]
 	} else {
 	    set pctbusy 100
 	}
@@ -420,7 +420,7 @@ proc redraw_new_style {} {
 	    .top.busy$index configure -background #C00000 -foreground "#000000"
 	}
 
-	.top.busy$index configure -text "$busy/$totalslaves\n$pctbusy%"
+	.top.busy$index configure -text "$busy/$totalworkers\n$pctbusy%"
 
 	incr graph
 
@@ -503,8 +503,8 @@ proc redraw_new_style {} {
 	}
 	incr index
     }
-    lappend TotalData(busy) $busyslaves_total
-    lappend TotalData(total) $totalslaves_total
+    lappend TotalData(busy) $busyworkers_total
+    lappend TotalData(total) $totalworkers_total
     lappend TotalData(scanspersec) $scanspersec_total
     set scantime_avg [expr $scantime_total / ($scanspersec_total + 0.000000001)]
     set relayoktime_avg [expr $relayoktime_total / ($relayokspersec_total + 0.000000001)]
@@ -519,21 +519,21 @@ proc redraw_new_style {} {
     lappend TotalData(recipoktime) $recipoktime_avg
 
     set graph 0
-    set TotalData(busy) [graph [expr $graph / (1.0 * $num_graphs)] [expr ($graph + 1.0) / (1.0 * $num_graphs)] 0 $totalslaves_total $TotalData(busy) $index "Busy" $graph "#A00000" 1]
-    if {$totalslaves_total > 0} {
-	set busyslaves_total [expr int($busyslaves_total)]
-	set pctbusy [expr int((1.0 * $busyslaves_total) / (1.0 * $totalslaves_total) * 100)]
+    set TotalData(busy) [graph [expr $graph / (1.0 * $num_graphs)] [expr ($graph + 1.0) / (1.0 * $num_graphs)] 0 $totalworkers_total $TotalData(busy) $index "Busy" $graph "#A00000" 1]
+    if {$totalworkers_total > 0} {
+	set busyworkers_total [expr int($busyworkers_total)]
+	set pctbusy [expr int((1.0 * $busyworkers_total) / (1.0 * $totalworkers_total) * 100)]
     } else {
 	set pctbusy 100
     }
 
-    if {$totalslaves_active > 0} {
-	set busyslaves_active [expr int($busyslaves_active)]
-	set apct [expr int((1.0 * $busyslaves_active) / (1.0 * $totalslaves_active) * 100)]
+    if {$totalworkers_active > 0} {
+	set busyworkers_active [expr int($busyworkers_active)]
+	set apct [expr int((1.0 * $busyworkers_active) / (1.0 * $totalworkers_active) * 100)]
     } else {
 	set apct 100
     }
-    .top.busytotal configure -text "$busyslaves_total/$totalslaves_total\n$pctbusy%\n$busyslaves_active/$totalslaves_active\n$apct%"
+    .top.busytotal configure -text "$busyworkers_total/$totalworkers_total\n$pctbusy%\n$busyworkers_active/$totalworkers_active\n$apct%"
     incr graph
 
     if {$NewStyleShowScans} {
@@ -610,8 +610,8 @@ proc redraw {} {
     set after_token ""
     set index 0
     set persec_total 0
-    set busy_slaves_total 0
-    set avail_slaves_total 0
+    set busy_workers_total 0
+    set avail_workers_total 0
     set msgs_per_sec_total 0
     set ms_per_scan_total 0
     set num_machines 0
@@ -628,35 +628,35 @@ proc redraw {} {
 	}
 
 	set busy $Data($mach,busy_snap)
-	set total_slaves $Data($mach,total_slaves)
+	set total_workers $Data($mach,total_workers)
 	set msg0 $Data($mach,persec_snap)
 	set ms0 $Data($mach,time_snap)
 	set persec_total [expr $persec_total + $msg0]
-	# Format $busy to have as many characters as $total_slaves
-	set l [string length $total_slaves]
+	# Format $busy to have as many characters as $total_workers
+	set l [string length $total_workers]
 	set busy [format "%${l}d" $busy]
-	.top.busy$index configure -text "$busy/$total_slaves"
+	.top.busy$index configure -text "$busy/$total_workers"
 	.top.persec$index configure -text $msg0
 	.top.time$index configure -text $ms0
-	if {$busy == $total_slaves} {
+	if {$busy == $total_workers} {
 	    .top.name$index configure -background "#CCCC00"
 	} else {
 	    .top.name$index configure -background "#D9D9D9"
 	}
 
-	set Data($mach,busy) [graph 0 [expr 1.0/3] 0 $Data($mach,total_slaves) $Data($mach,busy) $index "Busy" 1 red 1]
+	set Data($mach,busy) [graph 0 [expr 1.0/3] 0 $Data($mach,total_workers) $Data($mach,busy) $index "Busy" 1 red 1]
 	set Data($mach,persec) [graph [expr 1.0/3] [expr 2.0/3] 0 auto $Data($mach,persec) $index "Msgs/Sec" 2 green 1]
 	set Data($mach,time) [graph [expr 2.0/3] 1 0 auto $Data($mach,time) $index "ms/scan" 3 blue 1]
 	incr index
 
 	if {$ms0 > 0 || $Data($mach,busy_snap) > 0 || $msg0 > 0} {
 	    incr num_machines
-	    incr busy_slaves_total $Data($mach,busy_snap)
-	    incr avail_slaves_total $Data($mach,total_slaves);
+	    incr busy_workers_total $Data($mach,busy_snap)
+	    incr avail_workers_total $Data($mach,total_workers);
 	    incr ms_per_scan_total $ms0
 	}
     }
-    lappend TotalData(busy) $busy_slaves_total
+    lappend TotalData(busy) $busy_workers_total
     if {$num_machines > 0} {
 	lappend TotalData(time) [expr 1.0 * $ms_per_scan_total / (1.0 * $num_machines)]
     } else {
@@ -665,7 +665,7 @@ proc redraw {} {
     lappend TotalData(persec) $persec_total
 
     incr index
-    set TotalData(busy) [graph 0 [expr 1.0/3] 0 $avail_slaves_total $TotalData(busy) $index "Busy" 1 red 1]
+    set TotalData(busy) [graph 0 [expr 1.0/3] 0 $avail_workers_total $TotalData(busy) $index "Busy" 1 red 1]
     set TotalData(persec) [graph [expr 1.0/3] [expr 2.0/3] 0 auto $TotalData(persec) $index "Msgs/Sec" 2 green 1]
     set TotalData(time) [graph [expr 2.0/3] 1 0 auto $TotalData(time) $index "ms/scan" 3 blue 1]
 
@@ -674,9 +674,9 @@ proc redraw {} {
     set day  [human_number [expr $persec_total * 86400.0]]
     set persec_total [strip_zeros [format "%.1f" $persec_total]]
     .top.c configure -text "Total throughput $persec_total/s = $hour/hour = $day/day"
-    set l [string length $avail_slaves_total]
-    set busy_slaves_total [format "%${l}d" $busy_slaves_total]
-    .top.busytotal configure -text "$busy_slaves_total/$avail_slaves_total"
+    set l [string length $avail_workers_total]
+    set busy_workers_total [format "%${l}d" $busy_workers_total]
+    .top.busytotal configure -text "$busy_workers_total/$avail_workers_total"
     .top.persectotal configure -text [strip_zeros [format "%.1f" $msgs_per_sec_total]]
     if {$num_machines > 0} {
 	.top.avgtime configure -text [strip_zeros [format "%.0f" [expr 1.0 * $ms_per_scan_total / (1.0 * $num_machines)]]]
@@ -860,7 +860,7 @@ proc reconfigure_new_style {} {
     set col 2
     set canv_width 200
     label .top.name -text "Machine Name"
-    label .top.busy -text "Busy Slaves   " -foreground "#A00000"
+    label .top.busy -text "Busy Workers   " -foreground "#A00000"
     grid .top.name -row 0 -column 0 -sticky new
     grid .top.busy -row 0 -column 1 -sticky new
     catch { destroy .top.clabels}
@@ -1056,7 +1056,7 @@ proc reconfigure {} {
     catch { destroy .top.name }
 
     label .top.name -text "Machine Name"
-    label .top.busy -text "Busy Slaves" -foreground "#A00000"
+    label .top.busy -text "Busy Workers" -foreground "#A00000"
     label .top.persec -text "Msgs/s" -foreground "#00A000"
     label .top.time -text " ms/scan " -foreground "#0000A0"
     label .top.c -text ""
@@ -1120,52 +1120,53 @@ proc reconfigure {} {
     wm deiconify .top
 }
 
-proc busyslaves { mach } {
+proc busyworkers { mach } {
     global ctr
     global Data
     incr ctr
-    set w .slaves$ctr
+    set w .workers$ctr
     catch { destroy $w }
     toplevel $w
-    wm title $w "Busy slaves: $mach"
-    wm iconname $w "$mach slaves"
-    set Data($mach,busyslavewin) $w
+    wm title $w "Busy workers: $mach"
+    wm iconname $w "$mach workers"
+    set Data($mach,busyworkerwin) $w
 
-    # Open a new SSH connection for the busyslaves info
+    # Open a new SSH connection for the busyworkers info
     global SSHCommand
     global MD_MX_CTRL
     set hmach [host_plus_user $mach]
     set fp [open "| $SSHCommand $hmach $MD_MX_CTRL" "r+"]
     fconfigure $fp -blocking 0
-    fileevent $fp readable [list busyslaves_readable $mach $fp]
+    fileevent $fp readable [list busyworkers_readable $mach $fp]
 
-    tickle_busyslaves $mach $fp
+    tickle_busyworkers $mach $fp
 
     text $w.t -width 80 -height 35
     pack $w.t -side left -expand 1 -fill both
     $w.t tag bind pid <Enter> [list enter_pid $w.t]
     $w.t tag bind pid <Leave> [list leave_pid $w.t]
-    $w.t tag bind pid <ButtonPress-1> [list trace_slave $w.t $mach]
+    $w.t tag bind pid <ButtonPress-1> [list trace_worker $w.t $mach]
 }
 
-proc tickle_busyslaves { mach fp } {
+proc tickle_busyworkers { mach fp } {
     global Data
 
     catch {
 	set Data($mach,busydata) ""
+	# We have to use the old command for backware-compatibility.
 	puts $fp "busyslaves\nfoo_no_such_command"
 	flush $fp
     }
 }
 
-proc busyslaves_readable { mach fp } {
+proc busyworkers_readable { mach fp } {
     global Data
 
     gets $fp line
     if {"$line" == ""} {
 	if {[eof $fp]} {
 	    close $fp
-	    catch { destroy $Data($mach,busyslavewin) }
+	    catch { destroy $Data($mach,busyworkerwin) }
 	}
 	return
     }
@@ -1173,11 +1174,11 @@ proc busyslaves_readable { mach fp } {
 	lappend Data($mach,busydata) $line
 	return
     }
-    update_busyslaves $mach $fp
-    global BusySlaveUpdateInterval
-    after $BusySlaveUpdateInterval [list tickle_busyslaves $mach $fp]
+    update_busyworkers $mach $fp
+    global BusyWorkerUpdateInterval
+    after $BusyWorkerUpdateInterval [list tickle_busyworkers $mach $fp]
 }
-proc trace_slave { w mach } {
+proc trace_worker { w mach } {
     global TraceCommand
     set tags [$w tag names current]
     set index [lsearch -glob $tags "Z*"]
@@ -1205,7 +1206,7 @@ proc leave_pid { w } {
     }
 }
 
-proc compare_slaves { a b } {
+proc compare_workers { a b } {
     set acmd [lindex $a 3]
     set bcmd [lindex $b 3]
     set x [string compare $bcmd $acmd]
@@ -1230,9 +1231,9 @@ proc compare_slaves { a b } {
     return [expr $an - $bn]
 }
 
-proc update_busyslaves { mach fp} {
+proc update_busyworkers { mach fp} {
     global Data
-    set w $Data($mach,busyslavewin)
+    set w $Data($mach,busyworkerwin)
     if {![winfo exists $w]} {
 	catch { close $fp }
 	return
@@ -1247,7 +1248,7 @@ proc update_busyslaves { mach fp} {
 	}
     }
 
-    set busyguys [lsort -command compare_slaves $Data($mach,busydata)]
+    set busyguys [lsort -command compare_workers $Data($mach,busydata)]
 
     set count(scan) 0
     set count(relayok) 0
@@ -1256,13 +1257,13 @@ proc update_busyslaves { mach fp} {
 
     foreach line $busyguys {
 	set lst [split $line]
-	set slaveno [lindex $lst 0]
+	set workerno [lindex $lst 0]
 	set pid [lindex $lst 2]
 	set cmd [lindex $lst 3]
 	incr count($cmd)
-	set len [string length "$slaveno B $pid "]
+	set len [string length "$workerno B $pid "]
 	set line [string range $line $len end]
-	$w.t insert end [format "%4d" $slaveno] slaveno
+	$w.t insert end [format "%4d" $workerno] workerno
 	$w.t insert end " "
 	$w.t tag delete "Z$pid"
 	$w.t insert end [format "%6d" $pid] [list pid "Z$pid"]
@@ -1270,7 +1271,7 @@ proc update_busyslaves { mach fp} {
 
     }
 
-    set title "Busy slaves: $mach"
+    set title "Busy workers: $mach"
     foreach cmd {scan relayok senderok recipok} {
 	if {$count($cmd) > 0} {
 	    set c $count($cmd)
@@ -1284,7 +1285,7 @@ proc popup_machine_menu { m index x y} {
     catch { destroy .m }
     menu .m -tearoff 0
     .m add command -label "SSH" -command [list ssh $m]
-    .m add command -label "Busy Slaves" -command [list busyslaves $m]
+    .m add command -label "Busy Workers" -command [list busyworkers $m]
     .m add separator
     .m add command -label "Delete" -command [list del_machine $m]
     tk_popup .m $x $y
